/* 
file.c 
ALICE V 2.0, January 1993
library functions for file handling 
(C) Ralf Kaufmann 1993
used C compiler: Pure C 
*/ 
 
/* Includefiles */ 
 
#include <ctype.h> 
#include <gem.h> 
#include <osbind.h> 
#include <stdio.h> 
#include <stdlib.h> 
#include <string.h> 
#include "language.h" 
#include "prvdef.h" 
#include "mewidi.h" 
#include "messages.h" 
#include "scancode.h" 
#include "structs.h" 
#include "rsc.h"
 
 
/* References to global data defined elsewhere  */ 
 
extern  WIND_DATA   windows[]; 
extern  BUFFER      buffers[]; 
extern  BUFFER      include_buffer; 
extern  OBJECT      *dial_about, *dial_help, *dial_print, *dial_tab; 
extern  OBJECT      *dial_fread, *dial_fwrite; 
extern  WORD        page_length, left_margin; 
extern  WORD        tab_size; 
extern  char        question[];
extern  char        delete_name[]; 
extern  char        delete_path[]; 
extern  char        delete_filename[]; 

extern int backup;

// Funktionsprototypen
void add_element(LINE_DESC *e, int w_index);
void add_element_buffer(LINE_DESC *e, int b_index);
void add_element_include_buffer(LINE_DESC *e);
LINE_DESC *create_list_element(void);
void gem_exit(int rsrc); 
int init_buffer(int b_index);
int init_include_buffer(void);
long max(long a, long b);
int paste_include_file(int w_index);
void put_message(int msg_nr);
int rk_strrchr(char str[], char c);
void rk_strexp(char *p, int b);

/**********************/ 
/* Start of routines  */ 
/**********************/ 
 
 
 
 
int read_file(char *filename, int w_index) 
//char  *filename; 
//int   w_index;         /* index into global "windows" structure  */ 
{ 
  char        read_line   [MAX_LINE+1]; 
  int         error, i, tmp_char, nonprint_char; 
  LINE_DESC   *p, *next; 
 
 
  error = FALSE; 
  nonprint_char = FALSE;
  MOUSE_IS_BUSY; 
  if ((windows[w_index].fp = fopen (filename, "r")) == (FILE *) NULL) 
  { 
    put_message (ERR_OPEN_FILE); 
    return (FALSE);
  } 
  windows[w_index].doc_width  = 0; 
  windows[w_index].doc_length = 0; 
 
  for (i = 0; i < MAX_LINE; i++) read_line[i] = '\0';
  
  while (fgets (read_line, MAX_LINE-1, windows[w_index].fp) != NULL) 
                                                  /* read from file */ 
 
  {                                               /* 1. while start */ 
    if ((p = create_list_element ()) == NULL) 
    { 
      error = TRUE; 
      break;                                /* leave the while loop */ 
    }
    if (strchr (read_line, (int) '\t') != NULL)
    {
        rk_strexp (read_line, tab_size);            /* expand tabs  */
    }
 
    if ((p->line = (char *) malloc (MAX_LINE + 1)) == NULL) 
    { 
      error = TRUE; 
      free ((char *) p); 
      break;                                /* leave the while loop */ 
    } 
    memset (p->line, (int) 0, (size_t) (MAX_LINE-1));
    strcpy (p->line, read_line); 
    p->len  = (int) strlen (p->line); 
    p->size = MAX_LINE; 
		for (i = 0; i < p->len - 1; i++)	/* check nonprintable chars	*/
		{	
			tmp_char = (int) p->line[i];
			if (!(isascii(tmp_char) != 0) || (tmp_char == EOF)) continue;
			if (!(isprint (tmp_char))) nonprint_char = TRUE;
		}
    if (p->len > windows[w_index].doc_width) 
    { 
      windows[w_index].doc_width = p->len; 
    } 
    windows[w_index].doc_length++; 
 
    add_element (p, w_index);              /* add p to end of list */
    for (i = 0; i < MAX_LINE; i++) read_line[i] = '\0';
  }                                         /* 1. while end */
  if (windows[w_index].doc_length == 0 || 
      windows[w_index].doc_width  == 0) 
  {
    put_message (FILE_EMPTY);
    return (FALSE);   
  }
  if (error) 
  { 
    put_message (OUT_OF_MEMORY); 
    for (p = windows[w_index].head; p; p = next) 
    { 
      next = p->next; 
      free ((char *) p->line); 
      free ((LINE_DESC *) p);            /* free allocated memory  */ 
    } 
    windows[w_index].head = NULL;        /* init head of list      */ 
    fclose (windows[w_index].fp);        /* close the file         */ 
    windows[w_index].fp   = NULL; 
    return (FALSE); 
  } 
  else 
  { 
    fclose (windows[w_index].fp);                /* close the file */ 
    windows[w_index].fp = NULL; 
    if (nonprint_char) put_message (NONPRINTABLE_CHARACTERS);
    return (TRUE); 
  } 
}                                        /* end function read_file */ 
 
 

 
int read_file_buffer(char *filename, int b_index) 
//char  *filename; 
//int   b_index;         /* index into global "buffer" structure  */ 
{ 
  char        read_line   [MAX_LINE+1]; 
  int         error, i, line_count, tmp_char, nonprint_char; 
  LINE_DESC   *p, *next; 
  FILE        *fp;
 

  error = FALSE; 
  nonprint_char = FALSE;
  MOUSE_IS_BUSY; 
  line_count = 0;
  init_buffer (b_index);
  if ((fp = fopen (filename, "r")) == (FILE *) NULL) 
  { 
    put_message (ERR_OPEN_FILE); 
    return (FALSE);
  } 
  
  for (i = 0; i < MAX_LINE; i++) read_line[i] = '\0';
  
  while (fgets (read_line, MAX_LINE-1, fp) != NULL) 
                                                  /* read from file */ 
  { 
                                                  /* 1. while start */ 
    line_count = 1;
    if ((p = create_list_element ()) == NULL) 
    { 
      error = TRUE; 
      break;                                /* leave the while loop */ 
    }
    if (strchr (read_line, (int) '\t') != NULL)
    {
        rk_strexp (read_line, tab_size);            /* expand tabs  */
    }
 
    if ((p->line = (char *) malloc (MAX_LINE + 1)) == NULL) 
    { 
      error = TRUE; 
      free ((char *) p); 
      break;                                /* leave the while loop */ 
    } 
    memset (p->line, (int) 0, (size_t) (MAX_LINE-1));
    strcpy (p->line, read_line); 
    p->len  = (int) strlen (p->line); 
    p->size = MAX_LINE; 
		for (i = 0; i < p->len - 1; i++)	/* check nonprintable chars	*/
		{	
			tmp_char = (int) p->line[i];
			if (!(isascii(tmp_char) != 0) || (tmp_char == EOF)) continue;
			if (!(isprint (tmp_char))) nonprint_char = TRUE;
		}
    add_element_buffer (p, b_index);       /* add p to end of list */
    for (i = 0; i < MAX_LINE; i++) read_line[i] = '\0';
  }                                         /* 1. while end */
  if (line_count == 0)
  {
    put_message (FILE_EMPTY);
    return (FALSE);   
  }
  if (error) 
  { 
    put_message (OUT_OF_MEMORY); 
    for (p = buffers[b_index].head; p; p = next) 
    { 
      next = p->next; 
      free ((char *) p->line); 
      free ((LINE_DESC *) p);            /* free allocated memory  */ 
    } 
    buffers[b_index].head = NULL;        /* init head of list      */ 
    fclose (fp);                         /* close the file         */ 
    return (FALSE); 
  } 
  else 
  { 
    fclose (fp);                          /* close the file */ 
    if (nonprint_char) put_message (NONPRINTABLE_CHARACTERS);
    return (TRUE); 
  } 
}                                 /* end function read_file_buffer */ 
 
 
 
int read_include_file(char *filename) 
//char  *filename; 
{ 
  char        read_line[MAX_LINE+1]; 
  int         error, i, line_count, tmp_char, nonprint_char; 
  LINE_DESC   *p, *next; 
  FILE        *fp;
 

  error = FALSE; 
	nonprint_char = FALSE;
  MOUSE_IS_BUSY; 
  line_count = 0;
  init_include_buffer ();
  if ((fp = fopen (filename, "r")) == (FILE *) NULL) 
  { 
    put_message (ERR_OPEN_FILE); 
    return (FALSE);
  } 
  
  for (i = 0; i < MAX_LINE; i++) read_line[i] = '\0';
  
  while (fgets (read_line, MAX_LINE-1, fp) != NULL) 
                                                  /* read from file */ 
  { 
                                                  /* 1. while start */ 
    line_count = 1;
    if ((p = create_list_element ()) == NULL) 
    { 
      error = TRUE; 
      break;                                /* leave the while loop */ 
    }
    if (strchr (read_line, (int) '\t') != NULL)
    {
        rk_strexp (read_line, tab_size);            /* expand tabs  */
    }
 
    if ((p->line = (char *) malloc (MAX_LINE + 1)) == NULL) 
    { 
      error = TRUE; 
      free ((char *) p); 
      break;                                /* leave the while loop */ 
    } 
    memset (p->line, (int) 0, (size_t) (MAX_LINE-1));
    strcpy (p->line, read_line); 
    p->len  = (int) strlen (p->line); 
    p->size = MAX_LINE; 
		for (i = 0; i < p->len - 1; i++)	/* check nonprintable chars	*/
		{	
			tmp_char = (int) p->line[i];
			if (!(isascii(tmp_char) != 0) || (tmp_char == EOF)) continue;
			if (!(isprint (tmp_char))) nonprint_char = TRUE;
		}
    add_element_include_buffer (p);       /* add p to end of list */
    for (i = 0; i < MAX_LINE; i++) read_line[i] = '\0';
  }                                         /* 1. while end */
  if (line_count == 0)
  {
    put_message (FILE_EMPTY);
    return (FALSE);   
  }
  if (error) 
  { 
    put_message (OUT_OF_MEMORY); 
    for (p = include_buffer.head; p; p = next) 
    { 
      next = p->next; 
      free ((char *) p->line); 
      free ((LINE_DESC *) p);            /* free allocated memory  */ 
    } 
    include_buffer.head = NULL;          /* init head of list      */ 
    fclose (fp);                         /* close the file         */ 
    return (FALSE); 
  } 
  else 
  { 
    fclose (fp);                          /* close the file */ 
    if (nonprint_char) put_message (NONPRINTABLE_CHARACTERS);
    return (TRUE); 
  } 
}                                 /* end function read_include_file */ 
 
 
 
/* creates a file and writes a blank into it */ 
 
int create_file(char *filename, int w_index) 
//char  *filename; 
//int   w_index; 
{ 

  if ((windows[w_index].fp = fopen (filename, "w")) == (FILE *) NULL) 
  { 
    put_message (ERR_CREATE_FILE); 
    return (FALSE); 
  } 
 
  if (fseek (windows[w_index].fp, 0L,0)) 
  { 
    put_message (ERR_AT_WRITE); 
    fclose (windows[w_index].fp);
    gem_exit (TRUE); 
  } 
  clearerr (windows[w_index].fp); 
  fprintf (windows[w_index].fp, "%s", " "); /* write a blank */ 
  if (ferror(windows[w_index].fp)) 
  { 
    put_message (ERR_AT_WRITE); 
    fclose (windows[w_index].fp);
    gem_exit (TRUE); 
  } 
  if (fclose(windows[w_index].fp)) 
  { 
    put_message (ERR_AT_CLOSE); 
    gem_exit (TRUE); 
  } 
  windows[w_index].fp = NULL; 
  return (TRUE); 
}                                   /* end function create_file */ 
 
 
 
 
/* opens an existing file (or creates a new file) on disk   */ 
/* writes window buffer into disk file                      */ 
 
int write_file(char *filename, int w_index) 
//char  *filename; 
//int   w_index;    /* index into global "windows" structure  */ 
{ 
  LINE_DESC   *p, *next; 
 
 
  MOUSE_IS_BUSY;
  if ((windows[w_index].fp = fopen (filename, "w")) == (FILE *) NULL) 
  { 
    return (FALSE); 
  } 
 
  if (fseek (windows[w_index].fp, 0L,0)) 
  { 
    put_message (ERR_AT_WRITE); 
    fclose (windows[w_index].fp);
    gem_exit (TRUE); 
  } 
 
  for (p = windows[w_index].head; p; p = next) 
  { 
    next = p->next; 
    clearerr (windows[w_index].fp); 
    fprintf (windows[w_index].fp, "%s", p->line); 
    if (ferror(windows[w_index].fp)) 
    { 
      put_message (ERR_AT_WRITE); 
      fclose (windows[w_index].fp);
      gem_exit (TRUE); 
    } 
  } 
  if (fclose(windows[w_index].fp)) 
  { 
    put_message (ERR_AT_CLOSE); 
    gem_exit (TRUE); 
  } 
  windows[w_index].fp = NULL; 
  return (TRUE); 
}                                       /* end function write_file  */ 
 
 
 
 
/* routine for deleting a file  */ 
 
int delete_file(void) 
{ 
  int   length, pos, rtc, /*exit_button,*/ offset; 
  short	exit_button;
  FILE  *fp; 
 
 
  delete_path[0] = 'A'; 
  delete_path[0] += Dgetdrv(); 
  delete_path[1] = ':'; 
  Dgetpath (delete_path + 2, 0); 
  strcpy (delete_path + (int) strlen (delete_path), "\\*.*"); 
  strcpy (delete_name, "");
  
  strcpy (delete_name, DELETE_FILE);  /* make sure the user is      */
                                      /* aware of the operation     */
  if (!fsel_input (delete_path, 
                   delete_name, 
                   &exit_button)) return (FALSE);          /* error */ 
  if (exit_button == 0) return (TRUE);          /* "cancel" option  */ 
 
  /* do not perform the syntax checks on the file name here as for  */ 
  /* "open/backup as/save as": this enables you to actually delete  */ 
  /* files like ".x" or "." which are not shown in the file select  */ 
  /* box but which might exist on the disk. the worst case is that  */ 
  /* that the file is not found: so don't worry, be happy...        */ 
 
 
  /* assemble the complete filename */ 
 
  strcpy (delete_filename, delete_path); 
  offset = (int) max ((long) 0, 
                      (long) rk_strrchr (delete_filename, '\\')); 
                       /* get the position of  the last '\' */ 
  strcpy (delete_filename + offset, delete_name); 
 
  MOUSE_IS_BUSY; 
 
  if (Fsfirst (delete_filename, 0) < 0)          /* file not found */ 
  { 
    put_message (FILE_NOT_FOUND);
    return (FALSE); 
  } 

# ifdef english
  strcpy (question, "[2][ Really delete | "); 
# endif
# ifdef german
  strcpy (question, "[2][ Datei | "); 
# endif
# ifdef french
  strcpy (question, "[2][ Vraiement effacer | "); 
# endif

  length = (int) strlen (question); 
  if (strlen (delete_filename) <= ALERT_MAX_LINE_LENGTH) 
  { 
     strcpy (question + length, delete_filename); 
  } 
  else 
                    /* file name only if path is too long */ 
  { 
    pos = rk_strrchr (delete_filename, '\\'); 
    strcpy (question + length, delete_filename + pos); 
  } 
  length = (int) strlen (question); 
 
# ifdef english
  strcpy (question + length, " | now ? ][ Yes | No ]"); 
# endif
# ifdef german
  strcpy (question + length, 
          " | wirklich lschen ? ][ Ja | Nein ]"); 
# endif
# ifdef french
  strcpy (question + length, 
          " | maintenant ? ][ Oui | Non ]"); 
# endif

  MOUSE_IS_ARROW; 
  if (form_alert (1, question) == 2) return (TRUE);
  MOUSE_IS_BUSY;
  if ((fp = fopen (delete_filename, "w")) == (FILE *) NULL) 
  { 
    put_message (FILE_NOT_DELETED);
    return (FALSE); 
  } 
  fclose (fp); 
  rtc = Fdelete (delete_filename); 
  MOUSE_IS_ARROW; 
  if (rtc < 0)                                    /* error occurred */ 
  { 
    put_message (FILE_NOT_DELETED);
    return (FALSE); 
  } 
  else                                                  /* success  */ 
  { 
    put_message (FILE_DELETED);
  } 
 
 
  /* update default drive/path */ 
 
  Dsetdrv (delete_filename[0] - 'A'); 
  offset = 
    (int) max ((long) 0, (long) rk_strrchr (delete_path, '\\')); 
                             /* get the position of  the last '\' */ 
  strcpy (delete_path + offset, "\0"); 
  Dsetpath (delete_path); 
  return (TRUE); 
}                                 /* end of function delete_file  */ 
 

 
/* end of file file.c  */ 
 
