/* 
mouse.c 
ALICE V 2.0, January 1993
library functions for mouse handling 
(C) Ralf Kaufmann 1993 used C compiler: Pure C 
2003 Prof. Walz adaption to GnuC++ 2.9.5
*/ 
 
/* Includefiles */ 
 
#include <gem.h> 
#include <stdio.h> 
#include "language.h" 
#include "prvdef.h" 
#include "mewidi.h" 
#include "messages.h" 
#include "scancode.h" 
#include "structs.h" 
#include "rsc.h" 
 
 
 
 
/* References to global data defined elsewhere  */ 
 
extern  WIND_DATA   windows[]; 
extern  short       VDIhandle; 
extern  short				hzeich, bzeich; 
extern  OBJECT      *menu_tree; 
extern  int         showcr, closeall, prgquit;
 
// Funktionsprototypen
void cap_check(int w_index); 
void cap_enable(int flag); 
void clear_block_flags(short w_index); 
short cursor(short w_handle, short mode); 
short full_redraw(short w_handle, short clear); 
int get_free_index(void); 
short get_index_from_handle(short w_handle); 
int line_length(int  w_index, long number); 
long max(long a, long b);
long min(long a, long b);
int own_top_wind(void); 
void refresh_window(int w_handle);
void set_slider_pos(short w_handle);
void set_slider_size(short w_handle); 
int top_wind(void); 

/**********************/ 
/* Start of routines  */ 
/**********************/ 
 
 
 
 
int handle_mouse(int x, int y, int button) 
//int   x,y, button; 
{ 
  int   w_index, w_handle; 
  long  a, b, curs_x, curs_y, curs_x2, curs_y2; 
  int   jmc, slider; 
  int		x1, y1, x2, y2/*, gr_rlastwidth, gr_rlastheight*/; 
  short	gr_rlastwidth, gr_rlastheight; 
  
  int   ev_bclicks, ev_bmask, ev_bstate; 
  short	ev_bmx, ev_bmy, ev_bbutton, ev_bkstate; 

  LINE_DESC   *p, *next; 
 
 
  if (closeall || prgquit) return (TRUE); 
  if ((w_handle = top_wind()) == -1) return (TRUE); 
  if (!own_top_wind()) return (TRUE);
  w_index = get_index_from_handle(w_handle); 
  if (button & RIGHT_MOBUTTON)    /* is the right button pressed ?  */ 
  { 
	  graf_mouse(M_OFF, NULL);
    cursor(w_handle, 0); 

    /* recalc doc width and length before setting the sliders  */  

    windows[w_index].doc_width  = 0;
    windows[w_index].doc_length = 0;
    for (p = windows[w_index].head; p != NULL; p = next) 
    { 
      next = p->next; 
      if (p->len * bzeich > windows[w_index].doc_width) 
      { 
        windows[w_index].doc_width = p->len * bzeich; 
      } 
      windows[w_index].doc_length += hzeich; 
    } 
    clear_block_flags (w_index);
    cap_check (w_index);
    full_redraw (w_handle, TRUE);     /* refresh the window contents */ 
    menu_ienable (menu_tree, IOPEN, (get_free_index () != -1)); 
                                  /* check for more window handles  */
    cursor (w_handle, 1); 
  	graf_mouse(M_ON, NULL);
    return (TRUE); 
  } 
  if ((x < windows[w_index].work.g_x)                             || 
      (x > windows[w_index].work.g_x + windows[w_index].work.g_w) || 
      (y < windows[w_index].work.g_y)                             || 
      (y > windows[w_index].work.g_y + windows[w_index].work.g_h)) 
  { 
    ev_bclicks   = 1;        /* one mouseclick required for event  */ 
    ev_bmask     = 1;        /* wait for left button...            */ 
    ev_bstate    = 0;        /* ...to be released                  */ 
    
    evnt_button (ev_bclicks,          /* wait for the left button   */
                 ev_bmask, ev_bstate, /* to be released             */
                 &ev_bmx, &ev_bmy, 
                 &ev_bbutton, &ev_bkstate );

    return (TRUE); 
  } 
  
  x1 = x;
  y1 = y;
  gr_rlastwidth   = 0;
  gr_rlastheight  = 0;

  MOUSE_IS_POINT_HAND;  
  graf_rubbbox (x, y,             /* display a rubber box for     */
                0, 0,             /* block selection with the     */
                &gr_rlastwidth,   /* mouse. function returns on   */
                &gr_rlastheight );/* release of the mouse button  */
  MOUSE_IS_ARROW;
  x2  = x1 + gr_rlastwidth;
  x2  = (int)min(windows[w_index].work.g_x + windows[w_index].work.g_w,(long)x2);
  x2--; 
  y2  = y1 + gr_rlastheight;
  y2  = (int)min(windows[w_index].work.g_y + windows[w_index].work.g_h,(long)y2); 
  y2--;
  jmc     = TRUE; 
  slider  = FALSE; 
  curs_y  = windows[w_index].doc_y / hzeich + 1; 
  a       = max ((long) (y - windows[w_index].work.g_y), (long) 0); 
  a      -= a % hzeich;                             							/* truncate */ 
  curs_y  = curs_y + a / hzeich; 
  if (windows[w_index].doc_length / hzeich < curs_y) 
  { 
    curs_y = windows[w_index].doc_length / hzeich; 
  } 
  curs_x  = windows[w_index].doc_x / bzeich + 1; 
  b       = max ((long) (x - windows[w_index].work.g_x), (long) 0); 
  b      -= b % bzeich;                             /* truncate */ 
  curs_x  = curs_x + b / bzeich; 
  if (line_length (w_index, curs_y) < curs_x) 
  { 
    curs_x = line_length (w_index, curs_y); 
  } 
  graf_mouse(M_OFF, NULL);
  cursor (w_handle, 0); 
  windows[w_index].curs_x   = curs_x; 
  windows[w_index].curs_xs  = curs_x; 
  windows[w_index].curs_y   = curs_y; 
  if (windows[w_index].curs_x < windows[w_index].doc_x / bzeich + 1) 
  { 
    windows[w_index].doc_x  = 
      curs_x - windows[w_index].work.g_w / bzeich; 
    windows[w_index].doc_x  *= bzeich; 
    windows[w_index].doc_x  = 
      max ((long) windows[w_index].doc_x, (long) 0); 
    jmc     = FALSE; 
    slider  = TRUE; 
  } 

  curs_y2 = windows[w_index].doc_y / hzeich + 1; 
  a       = max ((long) (y2 - windows[w_index].work.g_y), (long) 0); 
  a       -= a % hzeich;                             /* truncate */ 
  curs_y2 = curs_y2 + a / hzeich; 
  if (windows[w_index].doc_length / hzeich < curs_y2) 
  { 
    curs_y2 = windows[w_index].doc_length / hzeich; 
  } 
  curs_x2 = windows[w_index].doc_x / bzeich + 1; 
  b       = max ((long) (x2 - windows[w_index].work.g_x), (long) 0); 
  b       -= b % bzeich;                             /* truncate */ 
  curs_x2 = curs_x2 + b / bzeich; 
  if (line_length (w_index, curs_y2) < curs_x2) 
  { 
    curs_x2 = line_length (w_index, curs_y2); 
  } 
  if (curs_x2 > curs_x || curs_y2 > curs_y)   /* block selection  */  
  {                                           /* with the mouse   */
    windows[w_index].curs_x = 1; 
    windows[w_index].curs_y = curs_y;
    if (windows[w_index].curs_x < windows[w_index].doc_x / bzeich + 1) 
    { 
      windows[w_index].doc_x  = 0;
      jmc     = FALSE; 
      slider  = TRUE; 
    } 
    windows[w_index].block.cs_x = 1; 
    windows[w_index].block.cs_y = curs_y; 
    if (curs_y2 + 1 <= windows[w_index].doc_length / hzeich)
    {
      windows[w_index].block.ce_x = 1; 
      windows[w_index].block.ce_y = curs_y2 + 1; 
    }
    else
    {
      windows[w_index].block.ce_x = line_length (w_index, curs_y2); 
      windows[w_index].block.ce_y = curs_y2; 
    }
    windows[w_index].block.set  = TRUE; 
    cap_enable (TRUE); 
    menu_ienable (menu_tree, IREPL, FALSE); 
    menu_ienable (menu_tree, IRDFILFI, FALSE); 
    menu_ienable (menu_tree, IBLPASTE, FALSE); 
    if (!jmc)
    {
      full_redraw (w_handle, TRUE);
    }
    else
    {
      full_redraw (w_handle, FALSE);
    }
    cursor (w_handle, 1); 
  	graf_mouse(M_ON, NULL);
    if (!windows[w_index].bg_modified) return (TRUE);  
  }
  if (slider) 
  { 
    set_slider_size (w_handle); 
    set_slider_pos (w_handle); 
  } 
  if (!jmc || windows[w_index].bg_modified) 
  { 
    refresh_window (w_handle); 
    windows[w_index].bg_modified = FALSE;
  } 
  cursor(w_handle, 1); 
  graf_mouse(M_ON, NULL);
  return (TRUE); 
}                                     /* end function handle_mouse  */ 
 
 
/* end of file mouse.c */ 
 
